use crate::database::controller::connector::establish_connection;
use crate::database::controller::roles::{
    add_permission, add_permission_context, get_role_permission_id, get_roles,
};
use crate::helper::check_access::check_access;
use crate::helper::settings::Settings;
use crate::modules::api::groups::create::GroupRolePermission;
use crate::schema::roles_permissions_context::dsl::roles_permissions_context;
use diesel::pg::types::sql_types::Uuid;
use diesel::result::Error;
use diesel::sql_types::Text;
use diesel::{sql_query, RunQueryDsl};
use rocket::State;

/// Execute add_permission_context for given group_id and permission
/// Warning: Will return Ok, even if permission context wasn't added due to missing permission for role
pub fn add_group_role_permission(
    settings: &State<Settings>,
    group_id: uuid::Uuid,
    role_id: String,
    permission: &str,
) -> Result<(), diesel::result::Error> {
    let role_permission_id = get_role_permission_id(settings, role_id, permission);
    match role_permission_id {
        Some(id) => match add_permission_context(settings, id, group_id) {
            Ok(_) => return Ok(()),
            Err(e) => return Err(e),
        },
        None => return Ok(()),
    }
}

pub fn add_group_role_permissions(
    settings: &State<Settings>,
    group_id: uuid::Uuid,
    permissions: Vec<GroupRolePermission>,
) -> Result<(), diesel::result::Error> {
    for permission in permissions {
        if permission.role_id != "admin".to_string() {
            if permission.permission_groups_core_edit {
                add_group_role_permission(
                    settings,
                    group_id,
                    permission.role_id.clone(),
                    crate::permissions::modules::member_management::groups::core::EDIT,
                )?;
            }
            if permission.permission_groups_delete {
                add_group_role_permission(
                    settings,
                    group_id,
                    permission.role_id.clone(),
                    crate::permissions::modules::member_management::groups::DELETE,
                )?;
            }
            if permission.permission_groups_members_view {
                add_group_role_permission(
                    settings,
                    group_id,
                    permission.role_id.clone(),
                    crate::permissions::modules::member_management::groups::members::VIEW,
                )?;
            }
            if permission.permission_groups_members_edit {
                add_group_role_permission(
                    settings,
                    group_id,
                    permission.role_id.clone(),
                    crate::permissions::modules::member_management::groups::members::EDIT,
                )?;
            }
            if permission.permission_groups_permissions_view {
                add_group_role_permission(
                    settings,
                    group_id,
                    permission.role_id.clone(),
                    crate::permissions::modules::member_management::groups::permissions::VIEW,
                )?;
            }
            if permission.permission_groups_permissions_edit {
                add_group_role_permission(
                    settings,
                    group_id,
                    permission.role_id.clone(),
                    crate::permissions::modules::member_management::groups::permissions::EDIT,
                )?;
            }
        } else {
            //Manual overwrite for admin. Gives admin all permissions
            add_group_role_permission(
                settings,
                group_id,
                permission.role_id.clone(),
                crate::permissions::modules::member_management::groups::core::EDIT,
            )?;
            add_group_role_permission(
                settings,
                group_id,
                permission.role_id.clone(),
                crate::permissions::modules::member_management::groups::DELETE,
            )?;
            add_group_role_permission(
                settings,
                group_id,
                permission.role_id.clone(),
                crate::permissions::modules::member_management::groups::members::VIEW,
            )?;
            add_group_role_permission(
                settings,
                group_id,
                permission.role_id.clone(),
                crate::permissions::modules::member_management::groups::members::EDIT,
            )?;
            add_group_role_permission(
                settings,
                group_id,
                permission.role_id.clone(),
                crate::permissions::modules::member_management::groups::permissions::EDIT,
            )?;
            add_group_role_permission(
                settings,
                group_id,
                permission.role_id.clone(),
                crate::permissions::modules::member_management::groups::permissions::VIEW,
            )?;
        }
    }
    Ok(())
}

pub fn check_role_has_permission_on_entity(
    settings: &State<Settings>,
    role_id: &str,
    permission: &str,
    entity_id: uuid::Uuid,
) -> Result<bool, diesel::result::Error> {
    use crate::database::controller::members::TempQuery;
    let connection = establish_connection(settings);

    let permission_count: Result<TempQuery, diesel::result::Error> = sql_query(
        "SELECT COUNT(role_permission_id) FROM roles_permissions_context WHERE entity = $1 AND role_permission_id IN (SELECT role_permission_id FROM roles_permissions WHERE role_id = $2 AND permission_id = $3)",
    )
        .bind::<Uuid, _>(entity_id)
        .bind::<Text, _>(role_id)
        .bind::<Text, _>(permission)
        .get_result(&connection);

    match permission_count {
        Ok(count) => {
            if count.count > 0 {
                Ok(true)
            } else {
                Ok(false)
            }
        }
        Err(e) => {
            error!("Error while checking members permissions (context): {}", e);
            Err(e)
        }
    }
}

pub fn get_group_permission_for_role(
    settings: &State<Settings>,
    group_id: uuid::Uuid,
    role_id: String,
) -> Result<GroupRolePermission, diesel::result::Error> {
    Ok(GroupRolePermission {
        role_id: role_id.clone(),
        permission_groups_core_edit: check_role_has_permission_on_entity(
            settings,
            &role_id,
            crate::permissions::modules::member_management::groups::core::EDIT,
            group_id,
        )?,
        permission_groups_delete: check_role_has_permission_on_entity(
            settings,
            &role_id,
            crate::permissions::modules::member_management::groups::DELETE,
            group_id,
        )?,
        permission_groups_members_view: check_role_has_permission_on_entity(
            settings,
            &role_id,
            crate::permissions::modules::member_management::groups::members::VIEW,
            group_id,
        )?,
        permission_groups_members_edit: check_role_has_permission_on_entity(
            settings,
            &role_id,
            crate::permissions::modules::member_management::groups::members::EDIT,
            group_id,
        )?,
        permission_groups_permissions_view: check_role_has_permission_on_entity(
            settings,
            &role_id,
            crate::permissions::modules::member_management::groups::permissions::VIEW,
            group_id,
        )?,
        permission_groups_permissions_edit: check_role_has_permission_on_entity(
            settings,
            &role_id,
            crate::permissions::modules::member_management::groups::permissions::EDIT,
            group_id,
        )?,
    })
}

pub fn get_group_role_permissions(
    settings: &State<Settings>,
    group_id: uuid::Uuid,
) -> Result<Vec<GroupRolePermission>, diesel::result::Error> {
    let mut grp: Vec<GroupRolePermission> = vec![];

    let roles = get_roles(settings)?;

    for role in roles {
        match get_group_permission_for_role(settings, group_id, role.id) {
            Ok(sgrp) => grp.push(sgrp),
            Err(e) => return Err(e),
        }
    }

    Ok(grp)
}
