use crate::database::controller::connector::establish_connection;
use crate::database::model::member_communication::CommunicationTarget;
use crate::helper::settings::Settings;
use crate::modules::api::member_management::model::member_communication::CommunicationType;
use diesel::{ExpressionMethods, QueryDsl, RunQueryDsl};
use rocket::State;

pub fn get_member_communication(
    settings: &State<Settings>,
    member_entity_id: uuid::Uuid,
) -> Vec<CommunicationTarget> {
    use crate::schema::communication_targets::dsl::*;
    use crate::schema::communication_types::dsl::*;

    let connection = establish_connection(settings);

    let data: Result<Vec<CommunicationTarget>, diesel::result::Error> = communication_targets
        .inner_join(communication_types)
        .filter(entity_id.eq(member_entity_id))
        .select((target_id, entity_id, type_name, type_id, value, description))
        .load(&connection);

    match data {
        Ok(data) => data,
        Err(e) => {
            error!(
                "Couldn't get communication targets for {}: {}",
                member_entity_id, e
            );
            vec![]
        }
    }
}

pub fn get_communication_target(
    settings: &State<Settings>,
    target_id2: uuid::Uuid,
) -> Result<CommunicationTarget, diesel::result::Error> {
    use crate::schema::communication_targets::dsl::*;
    use crate::schema::communication_types::dsl::*;

    let connection = establish_connection(settings);

    let data: Result<CommunicationTarget, diesel::result::Error> = communication_targets
        .inner_join(communication_types)
        .filter(target_id.eq(target_id2))
        .select((target_id, entity_id, type_name, type_id, value, description)).get_result(&connection);

    match data {
        Ok(data) => Ok(data),
        Err(e) => {
            error!(
                "Couldn't get communication target: {}",
                e
            );
            Err(e)
        }
    }
}

pub fn get_member_communication_types(settings: &State<Settings>) -> Vec<CommunicationType> {
    use crate::schema::communication_types::dsl::*;

    let connection = establish_connection(settings);

    let data: Result<Vec<CommunicationType>, diesel::result::Error> = communication_types
        .select((type_id, type_name))
        .load(&connection);

    match data {
        Ok(data) => data,
        Err(e) => {
            error!("Couldn't get communication types: {}", e);
            vec![]
        }
    }
}

pub fn add_communication_target(
    settings: &State<Settings>,
    target: CommunicationTarget,
) -> Result<uuid::Uuid, diesel::result::Error> {
    use crate::schema::communication_targets::dsl::*;

    let connection = establish_connection(settings);
    let result = diesel::insert_into(communication_targets)
        .values((
            entity_id.eq(target.entity_id),
            com_type.eq(target.type_id),
            value.eq(target.value),
            description.eq(target.description),
        ))
        .returning(target_id)
        .get_result(&connection);

    result
}

pub fn remove_communication_target(
    settings: &State<Settings>,
    target_id_to_delete: uuid::Uuid,
) -> Result<usize, diesel::result::Error> {
    use crate::schema::communication_targets::dsl::*;

    let connection = establish_connection(settings);
    let result = diesel::delete(communication_targets.filter(target_id.eq(target_id_to_delete)))
        .execute(&connection);

    match result {
        Ok(result) => Ok(result),
        Err(e) => {
            error!(
                "Couldn't delete communication target {}: {}",
                target_id_to_delete, e
            );
            Err(e)
        }
    }
}
