use crate::database::controller::connector::establish_connection;
use crate::database::model::member_licenses::RawMemberLicense;
use crate::helper::settings::Settings;
use crate::modules::member_management::model::licenses::MemberLicense;
use diesel::pg::types::sql_types::Uuid;
use diesel::{sql_query, ExpressionMethods, RunQueryDsl};
use rocket::State;

pub fn get_member_licenses(
    settings: &State<Settings>,
    entity_id: uuid::Uuid,
) -> Vec<MemberLicense> {
    let connection = establish_connection(settings);

    let data: Result<Vec<RawMemberLicense>, diesel::result::Error> = sql_query(
        "SELECT * FROM license_categories as lc
LEFT JOIN licenses_members lm on lc.name = lm.license_name AND lm.member_id=$1 ORDER BY lc.name ASC;",
    )
    .bind::<Uuid, _>(entity_id)
    .load(&connection);

    match data {
        Ok(data) => {
            let mut output: Vec<MemberLicense> = Vec::new();

            for license in data {
                output.push(MemberLicense::from(license.clone()));
                trace!("LICENSE: {}", license.name);
            }

            output
        }
        Err(e) => {
            error!("Couldn't get licenses for member {} : {}", entity_id, e);
            vec![]
        }
    }
}

pub fn add_member_license(
    settings: &State<Settings>,
    member_id2: uuid::Uuid,
    license_name2: String,
) -> Result<usize, diesel::result::Error> {
    use crate::schema::licenses_members::dsl::*;

    let connection = establish_connection(settings);

    let result = diesel::insert_into(licenses_members)
        .values((
            member_id.eq(member_id2),
            license_name.eq(license_name2.clone()),
        ))
        .execute(&connection);

    match result {
        Ok(num) => Ok(num),
        Err(e) => {
            error!(
                "Couldn't add license {} for member {} : {}",
                license_name2, member_id2, e
            );
            Err(e)
        }
    }
}

pub fn add_member_driving_permission(
    settings: &State<Settings>,
    member_id2: uuid::Uuid,
    license_name2: String,
) -> Result<usize, diesel::result::Error> {
    use crate::schema::licenses_members::dsl::*;

    let connection = establish_connection(settings);

    let result = diesel::update(licenses_members)
        .filter(member_id.eq(member_id2))
        .filter(license_name.eq(license_name2.clone()))
        .set(drive_permission.eq(true))
        .execute(&connection);

    match result {
        Ok(num) => Ok(num),
        Err(e) => {
            error!(
                "Couldn't add drive_permission to license {} for member {} : {}",
                license_name2, member_id2, e
            );
            Err(e)
        }
    }
}

pub fn remove_member_license(
    settings: &State<Settings>,
    member_id2: uuid::Uuid,
    license_name2: String,
) -> Result<usize, diesel::result::Error> {
    use crate::schema::licenses_members::dsl::*;

    let connection = establish_connection(settings);

    let result = diesel::delete(licenses_members)
        .filter(member_id.eq(member_id2))
        .filter(license_name.eq(license_name2.clone()))
        .execute(&connection);

    match result {
        Ok(num) => Ok(num),
        Err(e) => {
            error!(
                "Couldn't remove license {} for member {} : {}",
                license_name2, member_id2, e
            );
            Err(e)
        }
    }
}
