use crate::database::controller::connector::establish_connection;
use crate::database::controller::member_insert::insert_address;
use crate::database::model::addresses::Address;
use crate::database::model::members::RawMember;
use crate::helper::forms::into_option;
use crate::helper::settings::Settings;
use crate::modules::member_management::model::member::{Member, MemberProfileForm};
use crate::schema::addresses::dsl::{addresses, id};
use crate::schema::members;
use chrono::NaiveDate;
use diesel::{ExpressionMethods, RunQueryDsl};
use rocket::State;
use std::str::FromStr;
use uuid::Uuid;
use iban::{Iban, ParseIbanError};

pub fn update_member(
    settings: &State<Settings>,
    member_form: MemberProfileForm,
    old_member_data: Member,
    old_address: Option<Address>,
    personal_profile_update: bool,
) -> bool {
    use crate::schema::members::columns::entity_id;

    let connection = establish_connection(settings);
    let member_id = old_member_data.entity_id.clone();
    let mut success = false;

    let new_address =
        match parse_profile_member_form_address(member_form.clone(), old_address.clone()) {
            Some(address) => address,
            None => return false,
        };

    match old_address {
        Some(old_address) => {
            let address_id_old = old_address.id.clone();
            let affected_rows_address = diesel::update(addresses)
                .filter(id.eq(address_id_old))
                .set(&new_address)
                .execute(&connection);
            match affected_rows_address {
                Ok(num) => {
                    if num != 1 {
                        error!(
                            "Updated address {}, but query affected {} rows!",
                            address_id_old, num
                        );
                        success = false;
                    } else {
                        success = true;
                    }
                }
                Err(e) => {
                    error!(
                        "Tried to update address {}, but query failed: {}",
                        address_id_old, e
                    );
                    success = false
                }
            }
        }
        None => {
            let result = insert_address(settings, new_address, member_id);
            if !result {
                success = false;
            }
        }
    }

    let new_member = match parse_profile_member_form_member(member_form.clone(), old_member_data, personal_profile_update) {
        Some(member) => member,
        None => return false,
    };

    let affected_rows_member = diesel::update(members::table)
        .filter(entity_id.eq(member_id))
        .set(&new_member)
        .execute(&connection);

    match affected_rows_member {
        Ok(num) => {
            if num != 1 {
                error!(
                    "Updated member {}, but query affected {} rows!",
                    member_id, num
                );
                success = false;
            } else {
                success = true;
            }
        }
        Err(e) => {
            error!(
                "Tried to update member {}, but query failed: {}",
                member_id, e
            );
            success = false
        }
    }

    success
}

fn parse_profile_member_form_address(
    member_form: MemberProfileForm,
    old_address: Option<Address>,
) -> Option<Address> {
    let address_id: uuid::Uuid = if old_address.is_some() {
        match Uuid::from_str(&member_form.address_id) {
            Ok(uuid) => {
                if uuid != old_address.clone().unwrap().id {
                    warn!(
                        "Submitted address_id doesn't match with old entity_id! {} vs {}",
                        uuid,
                        old_address.unwrap().id
                    );
                    return None;
                } else {
                    uuid
                }
            }
            Err(e) => {
                warn!(
                    "Couldn't parse address_id submitted via MemberProfileForm: {} {}",
                    member_form.address_id, e
                );
                return None;
            }
        }
    } else {
        Default::default()
    };

    let title = if old_address.is_some() {
        old_address.unwrap().title
    } else {
        None
    };

    Some(Address {
        id: address_id,
        title,
        street: Some(member_form.address_street),
        number: Some(member_form.address_number),
        zipcode: Some(member_form.address_zipcode),
        city: Some(member_form.address_city),
        country: Some(member_form.address_country),
    })
}

fn parse_profile_member_form_member(
    member_form: MemberProfileForm,
    old_member_data: Member,
    personnel_profile_change: bool,
) -> Option<RawMember> {
    let entity_id = match Uuid::from_str(&member_form.entity_id) {
        Ok(uuid) => {
            if uuid != old_member_data.entity_id {
                warn!(
                    "Submitted entity_id doesn't match with old entity_id! {} vs {}",
                    uuid, old_member_data.entity_id
                );
                return None;
            } else {
                uuid
            }
        }
        Err(e) => {
            warn!(
                "Couldn't parse entity_id submitted via MemberProfileForm: {}",
                e
            );
            return None;
        }
    };

    let date_of_birth = if member_form.date_of_birth.is_empty() {
        None
    } else {
        match NaiveDate::parse_from_str(&member_form.date_of_birth, "%Y-%m-%d") {
            Ok(date) => Some(date),
            Err(e) => {
                warn!(
                    "Couldn't parse date_of_birth submitted via MemberProfileForm: {}",
                    e
                );
                old_member_data.date_of_birth
            }
        }
    };

    let sex = if member_form.sex.is_empty() {
        None
    } else {
        match i16::from_str(&member_form.sex) {
            Ok(value) => match value {
                0 | 1 | 2 | 9 => Some(value),
                _ => old_member_data.sex,
            },
            Err(e) => {
                warn!("Couldn't parse sex submitted via MemberProfileForm: {}", e);
                old_member_data.sex
            }
        }
    };

    let personnel_number = match i32::from_str(&member_form.personnel_number) {
        Ok(value) => Some(value),
        Err(e) => {
            warn!(
                "Couldn't parse personnel_number submitted via MemberProfileForm: {}",
                e
            );
            old_member_data.personnel_number
        }
    };

    let entrance_date = if member_form.entrance_date.is_empty() {
        None
    } else {
        match NaiveDate::parse_from_str(&member_form.entrance_date, "%Y-%m-%d") {
            Ok(date) => Some(date),
            Err(e) => {
                warn!(
                    "Couldn't parse entrance_date submitted via MemberProfileForm: {}",
                    e
                );
                old_member_data.entrance_date
            }
        }
    };

    let iban = if member_form.iban.is_empty(){
        None
    }else{
        match member_form.iban.parse::<Iban>(){
            Ok(iban) => Some(iban.to_string()),
            Err(e) => {warn!("Couldn't parse iban submitted via MemberProfileForm: {}", e);
            old_member_data.iban
            }
        }
    };

    if(personnel_profile_change){
        Some(RawMember {
            entity_id,
            users_id: old_member_data.users_id,
            firstname: old_member_data.firstname,
            lastname: old_member_data.lastname,
            date_of_birth,
            sex: old_member_data.sex,
            salutation: old_member_data.salutation,
            place_of_birth: into_option(member_form.place_of_birth),
            academic_titles: old_member_data.academic_titles,
            personnel_number: old_member_data.personnel_number,
            ui_language: old_member_data.ui_language,
            nationality: into_option(member_form.nationality),
            entrance_date,
            birth_name: into_option(member_form.birth_name),
            iban,
            bic: into_option(member_form.bic),
        })
    }else{
        Some(RawMember {
            entity_id,
            users_id: old_member_data.users_id,
            firstname: member_form.firstname,
            lastname: member_form.lastname,
            date_of_birth,
            sex,
            salutation: into_option(member_form.salutation),
            place_of_birth: into_option(member_form.place_of_birth),
            academic_titles: into_option(member_form.academic_titles),
            personnel_number,
            ui_language: old_member_data.ui_language,
            nationality: into_option(member_form.nationality),
            entrance_date,
            birth_name: into_option(member_form.birth_name),
            iban,
            bic: into_option(member_form.bic),
        })
    }

}
