use crate::database::controller::connector::establish_connection;
use crate::database::model::roles::Role;
use crate::diesel::query_dsl::select_dsl::SelectDsl;
use crate::helper::settings::Settings;
use diesel::query_dsl::filter_dsl::FilterDsl;
use diesel::result::Error;
use diesel::{BoolExpressionMethods, ExpressionMethods, RunQueryDsl};
use rocket::State;

pub fn get_roles(settings: &State<Settings>) -> Result<Vec<Role>, diesel::result::Error> {
    use crate::schema::roles::dsl::*;

    let connection = establish_connection(settings);
    match roles.load::<Role>(&connection) {
        Ok(roles_list) => Ok(roles_list),
        Err(e) => {
            error!("Couldn't load roles: {}", e);
            Err(e)
        }
    }
}

pub fn get_role_permission_id(
    settings: &State<Settings>,
    role_id2: String,
    permission: &str,
) -> Option<uuid::Uuid> {
    use crate::schema::roles_permissions::dsl::*;

    let connection = establish_connection(settings);
    let rpi: Result<uuid::Uuid, diesel::result::Error> = roles_permissions
        .filter(role_id.eq(role_id2).and(permission_id.eq(permission)))
        .select(role_permission_id)
        .first(&connection);

    match rpi {
        Ok(rpi) => Some(rpi),
        Err(error) => match error {
            diesel::result::Error::NotFound => None,
            _ => {
                error!("Couldn't get role permission id: {}", error);
                None
            }
        },
    }
}

pub fn add_permission(
    settings: &State<Settings>,
    role_id2: String,
    permission: String,
) -> Result<uuid::Uuid, diesel::result::Error> {
    use crate::schema::roles_permissions::dsl::*;
    let connection = establish_connection(settings);

    let inserted = diesel::insert_into(roles_permissions)
        .values((role_id.eq(role_id2), permission_id.eq(permission)))
        .returning(role_permission_id)
        .get_result(&connection);

    match inserted {
        Ok(id) => Ok(id),
        Err(e) => {
            error!("Couldn't add permission: {}", e);
            Err(e)
        }
    }
}

pub fn add_permission_context(
    settings: &State<Settings>,
    role_permission_id2: uuid::Uuid,
    context: uuid::Uuid,
) -> Result<(), diesel::result::Error> {
    use crate::schema::roles_permissions_context::dsl::*;
    let connection = establish_connection(settings);

    let inserted = diesel::insert_into(roles_permissions_context)
        .values((
            role_permission_id.eq(role_permission_id2),
            entity.eq(context),
        ))
        .execute(&connection);
    match inserted {
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Couldn't add permission context: {}", e);
            Err(e)
        }
    }
}
