use crate::database::controller::connector::establish_connection;
use crate::database::model::users::User;
use crate::diesel::prelude::*;
use crate::diesel::QueryDsl;
use crate::helper::settings::Settings;
use diesel::ExpressionMethods;
use rocket::State;
use crate::schema::members::columns::entity_id;
use diesel::result::Error;
use crate::modules::member_management::model::login::Login;
use crate::schema::users::dsl::users;
use crate::schema::members::dsl::members;

pub fn get_user_by_email(email: String, settings: &State<Settings>) -> Option<User> {
    use crate::schema::users::dsl::email as email1;
    use crate::schema::users::dsl::users;

    let connection = establish_connection(settings);

    match users.filter(email1.eq(email)).first(&connection) {
        Ok(user) => Some(user),
        Err(_) => None,
    }
}

/// Tries to get user associated to member entity id
/// Returns:
/// * Ok<Some<User>> if user found
/// * Ok<None> if no user found for member entity id
/// * Err<diesel::result::Error> if database error occured
pub fn get_user_by_member(settings: &State<Settings>, member_uuid: uuid::Uuid) -> Result<Option<User>, diesel::result::Error>{
    use crate::schema::members::dsl::members;
    use crate::schema::users::dsl::*;

    let connection = establish_connection(settings);

    let user : Result<User, diesel::result::Error> = members.inner_join(crate::schema::users::table).filter(entity_id.eq(member_uuid)).select((id, password, email)).first(&connection);
    match user{
        Ok(user) => Ok(Some(user)),
        Err(e) => {
            match e{
                Error::NotFound => {Ok(None)}
                _ => {
                    error!("Couldn't get user for member: {}", e);
                    Err(e)
                }
            }
        }
    }
}

pub fn add_user_to_member(settings: &State<Settings>, member_uuid: uuid::Uuid, email: String) -> Result<User, diesel::result::Error>{
    let connection = establish_connection(settings);

    let user : Result<User, diesel::result::Error> = diesel::insert_into(users).values(crate::schema::users::email.eq(email)).get_result(&connection);

    match user{
        Ok(user) => {
            match diesel::update(members.filter(crate::schema::members::entity_id.eq(member_uuid))).set(crate::schema::members::users_id.eq(user.id)).execute(&connection){
                Ok(_) => Ok(user),
                Err(e) => {
                    error!("Couldn't add user to member: {}", e);
                    Err(e)
                }
            }
        }
        Err(e) => {
            error!("Couldn't create new user: {}", e);
            Err(e)
        }
    }
}

pub fn remove_user(settings: &State<Settings>, user_id: uuid::Uuid) -> Result<(), diesel::result::Error>{
    use crate::schema::users::id;
    let connection = establish_connection(settings);

    match diesel::delete(users.filter(id.eq(user_id))).execute(&connection){
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Couldn't delete user: {}", e);
            Err(e)
        }
    }
}

pub fn update_user_email(settings: &State<Settings>, user_id: uuid::Uuid, email: String) -> Result<User, diesel::result::Error>{

    let connection = establish_connection(settings);

    match diesel::update(users.filter(crate::schema::users::id.eq(user_id))).set(crate::schema::users::email.eq(email)).get_result(&connection){
        Ok(user) => Ok(user),
        Err(e) => {
            error!("Couldn't update user (id: {}) email: {}", user_id, e);
            Err(e)
        }
    }
}