use crate::database::controller::member_communication::{
    add_communication_target, get_member_communication_types, remove_communication_target,
};
use crate::database::model::member_communication::CommunicationTarget;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid};
use crate::modules::api::member_management::model::member_communication::{
    CommunicationTargetId, CommunicationTypeList,
};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};
use rocket::State;
use rocket_contrib::json::Json;

#[get("/api/member_management/get_communication_types", format = "json")]
pub fn api_member_get_communication_types(
    cookie: SessionCookie,
    settings: State<Settings>,
) -> Result<Json<CommunicationTypeList>, Json<ApiErrorWrapper>> {
    let member = parse_member_cookie(cookie.member)?;
    if !member.has_permission("modules.configuration.fields.communication_types.view".to_string()) {
        return Err(Json(
            ApiError::new(
                401,
                "Keine Berechtigung, Kommunikationstypen abzurufen!".to_string(),
            )
            .to_wrapper(),
        ));
    }

    //TODO: Proper error handling/display
    let type_list = get_member_communication_types(&settings);
    Ok(Json(CommunicationTypeList { types: type_list }))
}

#[post(
    "/api/member_management/add_communication_target",
    data = "<communication_target>",
    format = "json"
)]
pub fn api_member_add_communication_target(
    cookie: SessionCookie,
    settings: State<Settings>,
    communication_target: Json<CommunicationTarget>,
) -> Result<Json<CommunicationTargetId>, Json<ApiErrorWrapper>> {
    //TODO: Check permissions!
    let member = parse_member_cookie(cookie.member)?;

    match add_communication_target(&settings, communication_target.into_inner()) {
        Ok(target_id) => Ok(Json(CommunicationTargetId { target_id })),
        Err(_) => Err(Json(
            ApiError::new(500, "Datenbankfehler aufgetreten".to_string()).to_wrapper(),
        )),
    }
}

#[put(
    "/api/member_management/remove_communication_target?<target_id>",
    format = "json"
)]
pub fn api_member_remove_communication_target(
    cookie: SessionCookie,
    settings: State<Settings>,
    target_id: String,
) -> Result<(), Json<ApiErrorWrapper>> {
    //TODO: Check permissions!
    let member = parse_member_cookie(cookie.member)?;
    let target_id = parse_uuid(target_id)?;

    match remove_communication_target(&settings, target_id) {
        Ok(_) => Ok(()),
        Err(_) => Err(Json(
            ApiError::new(500, "Datenbankfehler aufgetreten".to_string()).to_wrapper(),
        )),
    }
}
