use crate::helper::session_cookies::model::SessionCookieStorage;
use crate::helper::settings::Settings;
use crate::helper::sitebuilder::model::alerts::{Alert, AlertClass};
use crate::modules::welcome::controller::login::{add_session_cookie, check_login};
use crate::modules::welcome::controller::render::get_context;
use crate::modules::welcome::model::login_form::LoginForm;
use rocket::http::Cookies;
use rocket::request::Form;
use rocket::response::Redirect;
use rocket::State;
use rocket_contrib::templates::Template;
use crate::modules::welcome::model::login_error_type::LoginError;
use crate::modules::welcome::controller::password_reset::request_password_reset;
use crate::helper::mail_templates::MailTemplates;
use crate::helper::mail_queue::queue::MailQueue;
use std::sync::Arc;
use crate::helper::sitebuilder::model::general::{Footer, Header, Stylesheet};
use crate::database::controller::password_resets::{validate_token, change_password_with_token, validate_password};

#[post("/", data = "<login_form>")]
pub fn welcome_post(
    login_form: Form<LoginForm>,
    settings: State<Settings>,
    cookie_storage: State<SessionCookieStorage>,
    cookies: Cookies,
) -> Result<Redirect, Template> {
    let user = check_login(login_form.into_inner(), &settings);

    let alert: Option<Alert> = match user {
        Ok(user) => {
            add_session_cookie(user, &settings, cookie_storage, cookies);
            return Ok(Redirect::to("/loginselect"));
        }
        Err(e) => match e{
            LoginError::UserNotFound => {
                Some(Alert::new(
                    AlertClass::Danger,
                    "Anmelden fehlgeschlagen! Email oder Passwort falsch!".to_string()))
            },
            LoginError::MaxLoginAttemptsExceeded => {
                Some(Alert::new(
                    AlertClass::Danger,
                    format!("Es wurden zu viele Anmeldeversuche durchgeführt. Der Account wurde für {} Minuten gesperrt! Bitte nutzen Sie ggf. die Passwort vergessen Funktion!", settings.application.login_lock_duration/60)))
            },
            LoginError::PasswordIncorrect => {
                Some(Alert::new(
                    AlertClass::Danger,
                    "Anmelden fehlgeschlagen! Email oder Passwort falsch!".to_string()))
            },
            LoginError::DatabaseError => {
                Some(Alert::new(
                    AlertClass::Danger,
                    "Es konnte keine Datenbankverbindung hergestellt werden!".to_string()))
            }
            LoginError::NoPassword => {
                Some(Alert::new(
                    AlertClass::Danger,
                    "Anmelden fehlgeschlagen! Email oder Passwort falsch!".to_string()))
            }
        }
    };

    Err(Template::render("module_welcome", &get_context(alert)))
}

#[derive(Serialize, Deserialize, FromForm)]
pub struct PasswordResetForm{
    pub(crate) email: String,
}

#[post("/password_reset", data = "<password_reset_form>")]
pub fn password_reset_post(
    password_reset_form: Form<PasswordResetForm>,
    settings: State<Settings>,
    mt: State<MailTemplates>,
    mq: State<Arc<MailQueue>>,
) -> Template {
    let mut alert : Option<Alert> = None;

    match request_password_reset(&settings, &mt, mq, password_reset_form.email.clone()){
        Ok(_) => {
            alert = Some(Alert::new(
                AlertClass::Success,
                "Falls ein Benutzer mit der angegebenen Email existiert wurde ein Link zum Zurücksetzen des Passwortes per Email verschickt!".to_string()))
        }
        Err(_) => {
            alert = Some(Alert::new(
                AlertClass::Danger,
                "Das Passwort konnte nicht zurückgesetzt werden. Bitte versuchen Sie es später erneut.".to_string()))

        }
    }
    Template::render("module_welcome", &get_context(alert))
}

#[derive(Serialize)]
pub struct PasswordResetPage{
    header: Header,
    footer: Footer,
    logo_path: String,
    alert: Option<Alert>,
}

#[get("/password_reset?<token>")]
pub fn password_change_get(
    settings: State<Settings>,
    token: String,
) -> Result<Template, Redirect> {
    match validate_token(&settings, token){
        Ok(_) => (),
        Err(_) => {
            return Err(Redirect::to("/?error=password_reset_token_invalid"))
        }
    }
    let header = Header {
        html_language: "de".to_string(),
        site_title: "einsatz.online".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer { scripts: vec![] };
    let render = PasswordResetPage{
        header,
        footer,
        logo_path: "/img/logo.jpg".to_string(),
        alert: None
    };

    Ok(Template::render("password_reset", &render))
}

#[derive(Serialize, Deserialize, FromForm)]
pub struct PasswordChangeForm{
    pub(crate) password: String,
}

#[post("/password_reset?<token>", data = "<password_change_form>")]
pub fn password_change_post(
    settings: State<Settings>,
    password_change_form: Form<PasswordChangeForm>,
    token: String,
) -> Result<Redirect, Template> {
    let mut alert = None;
    let password_change_form = password_change_form.into_inner();
    if !validate_password(password_change_form.password.clone()){
        alert = Some(Alert::new(AlertClass::Danger, "Das Passwort muss mindestens 10 Zeichen lang sein!".to_string()));
    }else {
        match change_password_with_token(&settings, token, password_change_form.password) {
            Ok(_) => {
                return Ok(Redirect::to("/?error=password_reset_success"))
            },
            Err(_) => {
                alert = Some(Alert::new(AlertClass::Danger, "Es ist ein interner Fehler aufgetreten, das Passwort konnte nicht geändert werden.".to_string()));
            }
        }
    }
    let header = Header {
        html_language: "de".to_string(),
        site_title: "einsatz.online".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer { scripts: vec![] };
    let render = PasswordResetPage{
        header,
        footer,
        logo_path: "/img/logo.jpg".to_string(),
        alert,
    };

    Err(Template::render("password_reset", &render))
}